/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef CAMERA_HPP
#define CAMERA_HPP

#include <v3.hpp>

namespace cpe
{
class p2d;

/** \brief Storage of a camera (position and z_near)*/
class camera
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  Constructor
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor */
        camera(const v3& position,const double& z_near=1.0);

        // ********************************************* //
        // ********************************************* //
        //  Access parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief get the center of the camera */
        const v3& get_center() const;
        /** \brief get the z_near value */
        const double& get_z_near() const;

        // ********************************************* //
        // ********************************************* //
        //  Coordinate transform
        // ********************************************* //
        // ********************************************* //

        /** \brief project a 3D position onto the camera plane */
        v3 project(const v3& p) const;
        /** \brief compute the image space coordinate given a position on the screen plane */
        static p2d real_to_image_coordinate(const v3& p,const unsigned int& N_x,const unsigned int& N_y);

    private:

        /** \brief internal camera center */
        v3 p;
        /** \brief internal z_near coordinate */
        double z_near;
};
}

#endif
