/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#ifndef RENDER_ENGINE_HPP
#define RENDER_ENGINE_HPP




namespace cpe
{

    class image_drawable_zbuffer;
    struct triangle;
    struct light;
    struct shading;
    struct camera;
    class p2d;
    class mesh;


    /** \brief Helper class to render 3D models */
    class render_engine
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  RENDER HELPER
        // ********************************************* //
        // ********************************************* //

        /** \brief render a 3D triangle
            \param triangle_shaded tri: the 3D triangle with normal & color information
            \param shading_parameters shading_param: the shading parameters
            \param light_parameters light: the light data (positions, ...)
            \param image_drawable_zbuffer *im: pointer to the image to draw on
        */
        static void render_triangle(const triangle& tri,const shading& shading_param,const light& light_param,const camera& camera_param,image_drawable_zbuffer *im);

        /** \brief render a 3D mesh
            \param mesh m: the 3D mesh with normal & color information
            \param shading_parameters shading_param: the shading parameters
            \param light_parameters light: the light data (positions, ...)
            \param image_drawable_zbuffer *im: pointer to the image to draw on
        */
        static void render_mesh(const mesh& m,const shading& shading_param,const light& light,const camera& camera_param,image_drawable_zbuffer *im);

    };
}

#endif
