/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <render_engine.hpp>
#include <image_drawable_zbuffer.hpp>
#include <shading.hpp>
#include <triangle.hpp>
#include <light.hpp>
#include <mesh.hpp>
#include <vertex_shader.hpp>
#include <vertex.hpp>
#include <camera.hpp>

#include <cmath>


namespace cpe
{

    void render_engine::render_triangle(const triangle& tri,const shading& shading_param,const light& light_param,const camera& camera_param,image_drawable_zbuffer *im)
    {
        //compute shaded color
        color c[3];
        for(unsigned int k=0;k<3;++k)
            c[k]=vertex_shader::compute_color(tri.v[k],shading_param,light_param,camera_param);


        //get 3D position
        const v3& x0=tri.v[0].p;
        const v3& x1=tri.v[1].p;
        const v3& x2=tri.v[2].p;

        //project the 3 vertices on the camera screen
        v3 x0p=camera_param.project(x0);
        v3 x1p=camera_param.project(x1);
        v3 x2p=camera_param.project(x2);

        //get the 2D camera coordinate
        p2d u0=camera::real_to_image_coordinate(x0p,im->get_Nx(),im->get_Ny());
        p2d u1=camera::real_to_image_coordinate(x1p,im->get_Nx(),im->get_Ny());
        p2d u2=camera::real_to_image_coordinate(x2p,im->get_Nx(),im->get_Ny());



        //send standard 2D triangle with color to the rendering process
        im->triangle(u0,u1,u2,c[0],c[1],c[2],x0.z(),x1.z(),x2.z());

    }

    void render_engine::render_mesh(const mesh& m,const shading& shading_param,const light& light_param,const camera& camera_param,image_drawable_zbuffer *im)
    {
        //pass through every triangles
        unsigned int N_triangle=m.number_of_triangle();
        for(unsigned int k_tri=0;k_tri<N_triangle;++k_tri)
        {
            triangle tri=m.get_triangle(k_tri);
            render_triangle(tri,shading_param,light_param,camera_param,im);
        }
    }

}
