/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef ALGORITHM_HPP
#define ALGORITHM_HPP

#include <p2d.hpp>
#include <vector>

namespace cpe
{

    struct pixel_interp;
    struct scanline_buffer;


    /** \brief Helper class to perform Bresenham and Scanline algorithm */
    class algorithm
    {
    public:

        /** \brief Bresenham algorithm between pixels u1 and u2
            \param p2d u1: starting pixel coordinates of the segment
            \param p2d u1: ending pixel coordinates of the segment
            \return vector of pixel-coordinates + relative position in the segment
        */
        static std::vector<pixel_interp> bresenham(const p2d& u1,const p2d& u2);


        /** \brief fill the scanline data buffer for a given pixel_interp vector
            \param std::vector<pixel_interp> vec: vector of pixel positions+relative position: given by the output of bresenham algorithm
            \param int line_number: edge number of the corresponding triangle
            \param int x_min: offset position for minimal x-position of the triangle
            \param scanline_buffer* buffer: pointer to the buffer of scanline data for y_min/y_max value
        */
        static void scanline_update(const std::vector<pixel_interp>& vec,const int& line_number,scanline_buffer* buffer);

    };





}


#endif
