/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef COLOR_HPP
#define COLOR_HPP

#include <iostream>
#include <exception_cpe.hpp>

namespace cpe
{
    /** \brief Container class for (r,g,b) color. Each component is an integer (in [0,255]) */
    class color
    {
    public:


        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor (0,0,0) */
        color();
        /** \brief direct constructor with gray value (r,b,g)=(gray,gray,gray) */
        color(const int& gray);
        /** \brief direct constructor with (r,b,g) value */
        color(const int& r,const int& g,const int& b);

        // ********************************************* //
        // ********************************************* //
        //  ACCESS COLOR VALUE
        // ********************************************* //
        // ********************************************* //

        /** \brief set color value */
        void set(const int& r,const int& g,const int& b);


        /** \brief get red value */
        const int& r() const;
        /** \brief get red value */
        int& r();
        /** \brief get green value */
        const int& g() const;
        /** \brief get green value */
        int& g();
        /** \brief get blue value */
        const int& b() const;
        /** \brief get blue value */
        int& b();

        // ********************************************* //
        // ********************************************* //
        //  COLOR INTERPOLATION
        // ********************************************* //
        // ********************************************* //

        /** \brief linear interpolation of colors
            \return (1-alpha)*c1 + alpha*c2
        */
        static color interpolate_linear(const color& c1,const color& c2,const double& alpha);

        /** \brief return the clamped version of the color (between 0 and 255) */
        color clamped() const;

        // ********************************************* //
        // ********************************************* //
        //  I/O
        // ********************************************* //
        // ********************************************* //

        /** \brief standard output */
        friend std::ostream& operator<<(std::ostream& stream,const color& c);


    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL STORAGE
        // ********************************************* //
        // ********************************************* //

        /** \brief internal red component */
        int internal_r;
        /** \brief internal green component */
        int internal_g;
        /** \brief internal blue component */
        int internal_b;

    };

    /** \brief Exception class for color */
    class exception_color: public exception_cpe
    {
    public:
        /** \brief empty constructor */
        exception_color():exception_cpe(){}
        /** \brief constructor with error information */
        exception_color(const std::string& msg,const std::string& file,const std::string& caller,const int& line):exception_cpe(msg,file,caller,line){}
    };
}

#endif
