/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef IMAGE_DRAWABLE_HPP
#define IMAGE_DRAWABLE_HPP


#include <image.hpp>
#include <p2d.hpp>
#include <color.hpp>
#include <algorithm.hpp>

#include <map>
#include <cmath>

namespace cpe
{


    /** \brief Specialized class of image where user can draw lines, triangles, ... */
    class image_drawable : public image
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        image_drawable();
        /** \brief create square picture of size NxN
            \param unsigned int N: size (width & height) of the picture
        */
        image_drawable(const unsigned int& N);
        /** \brief create picture of size Nx x Ny
            \param unsigned int Nx: picture width
            \param unsigned int Ny: picture height
        */
        image_drawable(const unsigned int& Nx,const unsigned int& Ny);
        /** \brief import pre-existing picture from file filename
            \param std::string filename: path to the file to load
        */
        image_drawable(const std::string& filename);


        // ********************************************* //
        // ********************************************* //
        //  DRAWING FUNCTIONS
        // ********************************************* //
        // ********************************************* //

        /** \brief draw line on image with uniform color */
        void line(const p2d& u1,const p2d& u2,const color& c=color(255,255,255));
        /** \brief draw line on image with interpolated color */
        void line(const p2d& u1,const p2d& u2,const color& c1,const color& c2);

        /** \brief draw triangle on image with uniform color */
        void triangle(const p2d& u1,const p2d& u2,const p2d& u3,const color& c=color(255,255,255));
        /** \brief draw line on image with interpolated color */
        void triangle(const p2d& u1,const p2d& u2,const p2d& u3,const color& c1,const color& c2,const color& c3);



    private:


    };



}





#endif
