/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/





#ifndef IMAGE_DRAWABLE_ZBUFFER_HPP
#define IMAGE_DRAWABLE_ZBUFFER_HPP

#include <image_drawable.hpp>
#include <z_buffer.hpp>

namespace cpe
{
    /** \brief Extension class from image_drawable in adding Z-buffer */
    class image_drawable_zbuffer : public image_drawable
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        image_drawable_zbuffer();
        /** \brief create square picture of size NxN
          \param unsigned int N: size (width & height) of the picture
      */
        image_drawable_zbuffer(const unsigned int& N);
        /** \brief create picture of size Nx x Ny
          \param unsigned int Nx: picture width
          \param unsigned int Ny: picture height
      */
        image_drawable_zbuffer(const unsigned int& Nx,const unsigned int& Ny);
        /** \brief import pre-existing picture from file filename
          \param std::string filename: path to the file to load
      */
        image_drawable_zbuffer(const std::string& filename);



        // ********************************************* //
        // ********************************************* //
        //  DRAWING FUNCTIONS
        // ********************************************* //
        // ********************************************* //

        /** \brief draw line on image with interpolated color and z-coordinate
          \param p2d u1,u2:    extreme positions of the segment in pixel-coordinates
          \param color c1,c2:  color values of the pixels
          \param double z1,z2: z-value of the pixels
      */
        void line(const p2d& u1,const p2d& u2,const color& c1,const color& c2,const double& z1,const double& z2);

        /** \brief draw triangle on image with interpolated color and z-coordinate
          \param p2d u1,u2,u3:    extreme positions of the triangle in pixel-coordinates
          \param color c1,c2,c3:  color values of the pixels
          \param double z1,z2,z3: z-value of the pixels
      */
        void triangle(const p2d& u1,const p2d& u2,const p2d& u3,const color& c1,const color& c2,const color& c3,const double& z1,const double& z2,const double& z3);


        // ********************************************* //
        // ********************************************* //
        //  I/O
        // ********************************************* //
        // ********************************************* //

        /** \brief export depth-buffer into a file */
        void export_z_buffer(const std::string& filename) const;

    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL STORAGE
        // ********************************************* //
        // ********************************************* //

        /** \brief Z-buffer storage based on z-coordinate

          The z-buffer has the same size than the image, and is initialized to +infinity
      */
        z_buffer internal_z_buffer;

    };
}

#endif
