/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef IMAGE_IO_HPP
#define IMAGE_IO_HPP

#include <string>

namespace cpe
{
class image;
class z_buffer;

/** \brief Helper class for I/O */
class image_io
{
    public:

        /** \brief Export the picture into ppm format (ascii)
            \param std::string filename: file to store the picture
            \param image pic: image to store
        */
        static void export_ppm(const std::string& filename,const image& pic);

        /** \brief Export the z-buffer as a gray picture into ppm format (ascii)
            \param std::string filename: file to store the picture
            \param z_buffer buffer: depth-map to store
        */
        static void export_ppm(const std::string& filename,const z_buffer& buffer);

        /** \brief read a ppm picture format
            \param std::string filename: path of the file to read
        */
        static image read_ppm(const std::string& filename);
};
}
#endif
