/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef SCANLINE_BUFFER_HPP
#define SCANLINE_BUFFER_HPP

#include <vector>
#include <scanline_data.hpp>

namespace cpe
{


/** \brief Container for scanline data */
class scanline_buffer
{
    public:

        /** \brief Constructor with (x_min,x_max) value
            \note the initialisation of the scanline_data is automatically performed to (+infinity/-infinity)
        */
        scanline_buffer(const unsigned int& x_min=0,const unsigned int& x_max=0);

        /** \brief size of the buffer */
        unsigned int size() const;

        /** \brief Accessor to an element of the buffer
            \note the x-coordinate is given as absolute (don't need to give relative to x_min)
        */
        const scanline_data& operator()(const int& current_x) const;
        /** \brief Affectation to an element of the buffer
            \note the x-coordinate is given as absolute (don't need to give relative to x_min)
        */
        scanline_data& operator()(const int& current_x);


    private:

        /** \brief check index to be in range of the vector */
        void check_index(const int& index) const;

        /** \brief store the offset in x to index to vector */
        int x_offset;

        /** \brief the internal data stored as a vector */
        std::vector<scanline_data> internal_data;
};
}

#endif
