/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/





#include <algorithm.hpp>
#include <scanline_buffer.hpp>

#include <cmath>
#include <pixel_interp.hpp>

namespace cpe
{
    std::vector<pixel_interp> algorithm::bresenham(const p2d& u1,const p2d& u2)
    {
        std::vector<pixel_interp> vec;
        // ********************************************* //
        // ********************************************* //
        //  TO DO: Algorithme de bresenham pour remplire vector<pixel_interp>
        // ********************************************* //
        // ********************************************* //
        vec.push_back(pixel_interp(u1,0.0));
        vec.push_back(pixel_interp(u2,1.0));

        return vec;
    }



    void algorithm::scanline_update(const std::vector<pixel_interp>& vec,const int& line_number,scanline_buffer* buffer)
    {

        // ********************************************* //
        // ********************************************* //
        // Algorithme de remplissage du buffer pour scanline
        //
        // Algorithme:
        //
        // |Pour tout v de vector<pixel_interp>
        // |
        // |  [x,y] <- position de v
        // |
        // |  y_buffer_min = buffer(x).first.y
        // |  y_buffer_max = buffer(x).second.y
        // |
        // |  si y_buffer_min > y
        // |    buffer.first <- [x,y] + alpha + line_number
        // |  si y_buffer_max < y
        // |    buffer.first <- [x,y] + alpha + line_number
        // |
        // |fin pour
        //
        // ********************************************* //
        // ********************************************* //


        scanline_buffer& b=*buffer;

        for(int k=0;k<static_cast<int>(vec.size());++k)
        {
            const pixel_interp& pixel=vec[k];
            const p2d& u=pixel.coord;

            scanline_data& old_data=b(u.x());

            if(u.y()>old_data.info_max.y)
            {
                old_data.info_max.y=u.y();
                old_data.info_max.alpha=pixel.alpha;
                old_data.info_max.line_index=line_number;
            }
            if(u.y()<old_data.info_min.y)
            {
                old_data.info_min.y=u.y();
                old_data.info_min.alpha=pixel.alpha;
                old_data.info_min.line_index=line_number;
            }


        }


    }

}
