/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <image.hpp>

#include <sstream>
#include <fstream>
#include <color.hpp>
#include <p2d.hpp>
#include <image_io.hpp>

namespace cpe
{

  
    image::image():internal_Nx(0),internal_Ny(0){}
    image::image(const unsigned int& N):internal_Nx(N),internal_Ny(N){internal_data.resize(4*N*N);}
    image::image(const unsigned int& Nx,const unsigned int& Ny):internal_Nx(Nx),internal_Ny(Ny){internal_data.resize(4*Nx*Ny);}
    image::image(const std::string& filename)
    {
        if(filename.find(".ppm")!=std::string::npos)
            *this=image_io::read_ppm(filename);
    }
    const unsigned int& image::get_Nx() const{return internal_Nx;}
    const unsigned int& image::get_Ny() const{return internal_Ny;}
    void image::resize(const unsigned int& N){*this=image(N);}
    void image::resize(const unsigned int& Nx,const unsigned int& Ny){*this=image(Nx,Ny);}
    void image::reshape(const unsigned int& Nx2)
    {
        unsigned int N_total=internal_Nx*internal_Ny;
        //check if size can be reshaped
        if(N_total%Nx2!=0)
        {
            //throw error
            std::ostringstream msg;msg<<"N_total cannot be reshaped with Nx2="<<Nx2;
            throw exception_image(msg.str(),__FILE__,__FUNCTION__,__LINE__);
        }

        //otherwise reshape
        internal_Nx=Nx2;
        internal_Ny=N_total/Nx2;
    }




    void image::fill(const color& c)
    {
        for(unsigned int k=0,N=internal_data.size();k<N;++k)
            internal_data[k]=c;
    }

    void image::fill_rectangle(const p2d& u1,const p2d& u2,const color& c)
    {

        //check position
        if(check_position(u1)==false || check_position(u2)==false)
        {
            //throw error
            std::ostringstream msg;msg<<"Picture has size "<<internal_Nx<<"x"<<internal_Ny<<" and you are trying to access element: ("<<u1<<"),("<<u2<<") outside of the picture";
            throw exception_image(msg.str(),__FILE__,__FUNCTION__,__LINE__);
        }

        //fill rectangle otherwise
        int x_min=0,y_min=0,x_max=0,y_max=0;
        if(u1.x()<u2.x()){x_min=u1.x();x_max=u2.x();}
        if(u1.y()<u2.y()){y_min=u1.y();y_max=u2.y();}
        for(int x=x_min;x<=x_max;++x)
        {
            for(int y=y_min;y<y_max;++y)
            {
                set(p2d(x,y),c);
            }
        }


    }

    int image::coordinate_to_index(const p2d& u) const
    {
        return u.x()+internal_Nx*u.y();
    }

    bool image::check_position(const p2d& u) const
    {
        return (u.x()<0 || u.x()>=static_cast<int>(internal_Nx) || u.y()<0 || u.y()>=static_cast<int>(internal_Ny))?false:true;
    }

    const color& image::operator()(const p2d& index) const
    {
        check_position(index);
        return internal_data[coordinate_to_index(index)];
    }
    color& image::operator()(const p2d& index)
    {
        check_position(index);
        return internal_data[coordinate_to_index(index)];
    }

    void image::set(const p2d& index,const color& c)
    {
        check_position(index);
        internal_data[coordinate_to_index(index)]=c;
    }
    void image::export_file(const std::string& filename)
    {
        if(filename.find(".ppm")!=std::string::npos)
            image_io::export_ppm(filename,*this);
    }

}
