/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <image_drawable.hpp>

#include <iostream>
#include <cmath>
#include <p2d.hpp>
#include <color.hpp>
#include <pixel_interp.hpp>
#include <scanline_buffer.hpp>

#include <map>
#include <cstdlib>



namespace cpe
{

    image_drawable::image_drawable():image(){}
    image_drawable::image_drawable(const unsigned int& N):image(N){}
    image_drawable::image_drawable(const unsigned int& Nx,const unsigned int& Ny):image(Nx,Ny){}
    image_drawable::image_drawable(const std::string& filename):image(filename){}


    //affiche une ligne de couleur uniforme
    void image_drawable::line(const p2d& u0,const p2d& u1,const color& c)
    {
        //calcul des pixels de la droite par l'algorithme de bresenham
        std::vector<pixel_interp> vec=algorithm::bresenham(u0,u1);

        //affichage de la droite: parcours de l'ensemble des pixels stockes
        for(unsigned int k=0,N=vec.size();k<N;++k)
        {
            //coordonnees 2D
            const p2d& u=vec[k].coord;

            //verification que les coordonnees sont bien dans les dimensions de l'image
            if(check_position(u)==true)
            {
                //modification de la couleur du pixel: "Image(u)=c"
                set(u,c);
            }
        }
    }

    //affiche une ligne dont la couleur est interpolee entre c1 et c2
    void image_drawable::line(const p2d& u1,const p2d& u2,const color& c1,const color& c2)
    {
        //calcul des pixels de la droite par l'algorithme de bresenham
        std::vector<pixel_interp> vec=algorithm::bresenham(u1,u2);

        //affichage de la droite: parcours de l'ensemble des pixels stockes
        for(unsigned int k=0,N=vec.size();k<N;++k)
        {
            //coordonnees 2D
            const p2d& u=vec[k].coord;

            //verification que les coordonnees sont bien dans les dimensions de l'image
            if(check_position(u)==true)
            {
                // ********************************************* //
                // ********************************************* //
                //  TO DO: Interpolation de la couleur
                //
                //
                // ********************************************* //
                // ********************************************* //
                color couleur_courante=color(0,0,0);

                //affectation de la couleur courante au pixel
                set(u,couleur_courante);
            }
        }

    }


    void image_drawable::triangle(const p2d& u1,const p2d& u2,const p2d& u3,const color& c)
    {

        // ********************************************* //
        // ********************************************* //
        //  TO DO: Tracé de triangle
        //
        // Algorithm:
        //
        // | vec[0] <- bresenham(u1,u2)
        // | vec[1] <- bresenham(u2,u3)
        // | vec[2] <- bresenham(u3,u1)
        // |
        // | (xmin,xmax) <- (min(u1.x,u2.x,u3.x),max(u1.x,u2.x,u3.x))
        // | buffer = scanline_buffer(xmin,xmax)
        // |
        // | Pour i=0:3
        // |   scanline_update(vec[i])
        // | fin pour
        // |
        // | Pour tout x contenu dans buffer
        // |   position_y_min <- recuperer info dans buffer(x)
        // |   position_y_max <- recuperer info dans buffer(x)
        // |   affiche_ligne(position_y_min,position_y_max,c)
        // | fin pour
        //
        // ********************************************* //
        // ********************************************* //

        line(u1,u2,c);
        line(u2,u3,c);
        line(u3,u1,c);

    }

    void image_drawable::triangle(const p2d& u1,const p2d& u2,const p2d& u3,const color& c1,const color& c2,const color& c3)
    {

        //color LUT
        const color* LUT_color_1[3];
        const color* LUT_color_2[3];
        LUT_color_1[0]=&c1;LUT_color_1[1]=&c2;LUT_color_1[2]=&c3;
        LUT_color_2[0]=&c2;LUT_color_2[1]=&c3;LUT_color_2[2]=&c1;


        // ********************************************* //
        // ********************************************* //
        //  TO DO: Tracé de triangle
        //
        // Algorithm:
        //
        // | vec[0] <- bresenham(u1,u2)
        // | vec[1] <- bresenham(u2,u3)
        // | vec[2] <- bresenham(u3,u1)
        // |
        // | (xmin,xmax) <- (min(u1.x,u2.x,u3.x),max(u1.x,u2.x,u3.x))
        // | buffer = scanline_buffer(xmin,xmax)
        // |
        // | Pour i=0:3
        // |   scanline_buffer(vec[i]);
        // | fin pour
        // |
        // | Pour tout x de size(scanline_buffer)
        // |
        // |   (u,alpha,line_index)_start <- recuperer info dans buffer(x)
        // |   (u,alpha,line_index)_end <- recuperer info dans buffer(x)
        // |
        // |   color_start <- interpole_couleur(*LUT_color_1[line_index_start],*LUT_color_2[line_index_start],alpha_start)
        // |   color_end   <- interpole_couleur(*LUT_color_1[line_index_end]  ,*LUT_color_2[line_index_end]  ,alpha_end)
        // |
        // |   affiche_ligne(u_start,u_end,color_start,color_end)
        // |
        // | fin pour
        //
        // ********************************************* //
        // ********************************************* //

        line(u1,u2,c1);
        line(u2,u3,c2);
        line(u3,u1,c3);



    }






}
