/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <image_drawable_zbuffer.hpp>
#include <pixel_interp.hpp>
#include <scanline_buffer.hpp>

namespace cpe
{
    image_drawable_zbuffer::image_drawable_zbuffer():image_drawable(){}
    image_drawable_zbuffer::image_drawable_zbuffer(const unsigned int& N):image_drawable(N),internal_z_buffer(N)
    {}
    image_drawable_zbuffer::image_drawable_zbuffer(const unsigned int& Nx,const unsigned int& Ny):image_drawable(Nx,Ny),internal_z_buffer(Nx,Ny)
    {}
    image_drawable_zbuffer::image_drawable_zbuffer(const std::string& filename):image_drawable(filename)
    {
        internal_z_buffer=z_buffer(get_Nx(),get_Ny());
    }
    void image_drawable_zbuffer::export_z_buffer(const std::string& filename) const
    {
        internal_z_buffer.export_file(filename);
    }

    void image_drawable_zbuffer::line(const p2d& u1,const p2d& u2,const color& c1,const color& c2,const double& z1,const double& z2)
    {

        // ********************************************* //
        // ********************************************* //
        //  TO DO: Tracé de lignes en couleur avec prise en compte et MAJ du Zbuffer
        //
        // Algorithm:
        //
        // | vecteur<pixel_interp> vec = bresenham(u1,u2)
        // |
        // | Pour tout v de vec
        // |
        // |   coord   <- v.coord
        // |   alpha   <- v.alpha
        // |   couleur <- interpolation_couleur(c1,c2,alpha)
        // |   z       <- interpolation_z(z1,z2,alpha)
        // |
        // |   if [ (z>0) & (z<Zbuffer(coord)) ]
        // |     Zbuffer(coord) = z
        // |     set_pixel(v.u , couleur , z)
        // |   fin if
        // |
        // | fin pour
        //
        // ********************************************* //
        // ********************************************* //

        image_drawable::line(u1,u2,color(0,0,0));
    }

    void image_drawable_zbuffer::triangle(const p2d& u1,const p2d& u2,const p2d& u3,const color& c1,const color& c2,const color& c3,const double& z1,const double& z2,const double& z3)
    {

        //color LUT
        const color* LUT_color_1[3];
        const color* LUT_color_2[3];
        LUT_color_1[0]=&c1;LUT_color_1[1]=&c2;LUT_color_1[2]=&c3;
        LUT_color_2[0]=&c2;LUT_color_2[1]=&c3;LUT_color_2[2]=&c1;


        //z-coordinate LUT
        const double* LUT_z_1[3];
        const double* LUT_z_2[3];
        LUT_z_1[0]=&z1;LUT_z_1[1]=&z2;LUT_z_1[2]=&z3;
        LUT_z_2[0]=&z2;LUT_z_2[1]=&z3;LUT_z_2[2]=&z1;


        // ********************************************* //
        // ********************************************* //
        //  TO DO: Tracé de triangle
        //
        // Algorithm:
        //
        // | vec[0] <- bresenham(u1,u2)
        // | vec[1] <- bresenham(u2,u3)
        // | vec[2] <- bresenham(u3,u1)
        // |
        // | (xmin,xmax) <- (min(u1.x,u2.x,u3.x),max(u1.x,u2.x,u3.x))
        // | buffer = scanline_buffer(xmin,xmax)
        // |
        // | Pour i=0:3
        // |   scanline_buffer(vec[i]);
        // | fin pour
        // |
        // | Pour tout x de size(scanline_buffer)
        // |
        // |   (u,alpha,line_index)_start <- recuperer info dans buffer(x)
        // |   (u,alpha,line_index)_end <- recuperer info dans buffer(x)
        // |
        // |   color_start <- interpole_couleur(*LUT_color_1[line_index_start],*LUT_color_2[line_index_start],alpha_start)
        // |   color_end   <- interpole_couleur(*LUT_color_1[line_index_end]  ,*LUT_color_2[line_index_end]  ,alpha_end)
        // |
        // |   z_start <- interpole_z(*LUT_z_1[line_index_start],*LUT_z_2[line_index_start],z_start)
        // |   z_end   <- interpole_z(*LUT_z_1[line_index_end]  ,*LUT_z_2[line_index_end]  ,z_end)
        // |
        // |   affiche_ligne(u_start,u_end,color_start,color_end,z_start,z_end)
        // |
        // | fin pour
        //
        // ********************************************* //
        // ********************************************* //

        image_drawable::line(u1,u2,c1);
        image_drawable::line(u2,u3,c2);
        image_drawable::line(u3,u1,c3);


    }




}
