/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <image_io.hpp>

#include <image.hpp>
#include <p2d.hpp>
#include <z_buffer.hpp>

#include <fstream>
#include <sstream>


namespace cpe
{
void image_io::export_ppm(const std::string& filename,const image& pic)
{
    std::ofstream stream(filename.c_str(),std::ofstream::out);
    if(!stream.good())
        throw exception_image("Error in opening file "+filename+"... : cannot open file ",__FILE__,__FUNCTION__,__LINE__);

    unsigned int Nx=pic.get_Nx();
    unsigned int Ny=pic.get_Ny();

    //magic number
    stream<<"P3 \n";
    //size
    stream<<Nx<<" "<<Ny<<" \n";
    //color number
    stream<<"255"<<std::endl;

    std::stringstream str;
    for(unsigned int k2=0;k2<Nx;++k2)
        for(unsigned int k1=0;k1<Ny;++k1)
            str<<pic(p2d(k1,k2))<<std::endl;

    stream<<str.str();

    stream.close();
}

void image_io::export_ppm(const std::string& filename,const z_buffer& buffer)
{
    std::ofstream stream(filename.c_str(),std::ofstream::out);
    if(!stream.good())
        throw exception_image("Error in opening file "+filename+"... : cannot open file ",__FILE__,__FUNCTION__,__LINE__);

    unsigned int Nx=buffer.size_x();
    unsigned int Ny=buffer.size_y();

    //magic number
    stream<<"P3 \n";
    //size
    stream<<Nx<<" "<<Ny<<" \n";
    //color number
    stream<<"255"<<std::endl;



    double z_min=buffer.get_min();
    double z_max=buffer.get_max();


    std::stringstream str;
    for(unsigned int k2=0;k2<Nx;++k2)
    {
        for(unsigned int k1=0;k1<Ny;++k1)
        {
            double z=buffer(p2d(k1,k2));
            double val=(z-z_min)/(z_max-z_min);
            color c=color(val*255,val*255,val*255).clamped();
            str<<c<<std::endl;
        }
    }

    stream<<str.str();

    stream.close();
}

image image_io::read_ppm(const std::string& filename)
{
    std::ifstream stream(filename.c_str());
    if(!stream.good())
        throw exception_image("Error in opening file ["+filename+"] ... : cannot open file ",__FILE__,__FUNCTION__,__LINE__);

    //go to magic number
    std::string buffer;
    buffer="#";
    while(buffer[0]=='#')
        std::getline(stream,buffer);
    if(buffer!="P3")
        throw exception_image("Cannot find P3 magic number in file "+filename,__FILE__,__FUNCTION__,__LINE__);

    //(Nx,Ny)
    buffer="#";
    while(buffer[0]=='#')
        std::getline(stream,buffer);
    std::istringstream tokens(buffer);

    unsigned int temp_Nx=0,temp_Ny=0;
    tokens>>temp_Nx;tokens>>temp_Ny;

    //color number
    stream>>buffer;

    image new_pic(temp_Nx,temp_Ny);

    std::vector<unsigned char> picture;
    for(unsigned int k_x=0;k_x<temp_Nx;k_x++)
    {
        for(unsigned int k_y=0;k_y<temp_Ny;k_y++)
        {
            unsigned char val[3];
            for(unsigned int k_dim=0;k_dim<3;++k_dim)
                stream>>val[k_dim];

            new_pic(p2d(k_x,k_y))=color(val[0],val[1],val[2]);
        }
    }


    stream.close();

    return new_pic;

}
}
