/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <scanline_buffer.hpp>
#include <exception_cpe.hpp>

#include <iostream>
#include <string>
#include <sstream>


namespace cpe
{
scanline_buffer::scanline_buffer(const unsigned int& x_min,const unsigned int& x_max)
    :x_offset(x_min),internal_data(x_max-x_min+1)
{
    for(unsigned int k=0,N=internal_data.size();k<N;++k)
        internal_data[k].x=k+x_min;
}

unsigned int scanline_buffer::size() const
{return internal_data.size();}


const scanline_data& scanline_buffer::operator()(const int& index) const
{
    //protection en cas de depassement de la taille du tableau
    check_index(index-x_offset);

    return internal_data[index-x_offset];
}
scanline_data& scanline_buffer::operator()(const int& index)
{
    //protection en cas de depassement de la taille du tableau
    check_index(index-x_offset);

    return internal_data[index-x_offset];
}
void scanline_buffer::check_index(const int& index) const
{
    if(index<0 || index>static_cast<int>(internal_data.size()))
    {
        std::stringstream msg;
        msg<<"Index "<<index+x_offset<<"not in range of scanline_buffer which is ["<<x_offset<<","<<x_offset+size()-1<<"]";

        throw exception_cpe(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
}

}
